//
//  tiledemoViewController.m
//  tiledemo
//
//  Created by Michael Heyeck on 3/13/10.
//  Copyright Fair Oaks Labs, Inc. 2010. All rights reserved.
//

#import "TileView.h"
#import "tiledemoViewController.h"


@interface tiledemoViewController ()

@property (nonatomic, retain) UIScrollView* scrollView;

- (void)removeTiles;
- (void)createTiles;

@end


@implementation tiledemoViewController

- (UIScrollView*)scrollView
{
	return (UIScrollView*) self.view;
}

- (void)setScrollView:(UIScrollView*)newScrollView
{
	self.view = newScrollView;
}


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
	
	rows = 25;
	cols = 50;
	[self createTiles];
	
	CGFloat w = self.scrollView.bounds.size.width;
	CGFloat h = self.scrollView.bounds.size.height;
	self.scrollView.contentSize = CGSizeMake(w*cols, h*rows);
	
	[self scrollViewDidScroll:self.scrollView];
}


- (void)dealloc
{
	// Clear weak delegate reference
	if (self.scrollView.delegate == self)
	{
		self.scrollView.delegate = nil;
	}
	
	[self removeTiles];
    [super dealloc];
}

#pragma mark Scroll view delegate methods

- (void)scrollViewDidScroll:(UIScrollView*)scrollView
{
	// Tile width and height (derived from scrollView)
	CGFloat w = self.scrollView.bounds.size.width;
	CGFloat h = self.scrollView.bounds.size.height;
	
	// Row and column of the UL tile
	NSInteger ul_tile_x = (NSInteger) self.scrollView.contentOffset.x/w;
	NSInteger ul_tile_y = (NSInteger) self.scrollView.contentOffset.y/h;
	ul_tile_x = MAX(MIN(ul_tile_x, cols-2), 0);
	ul_tile_y = MAX(MIN(ul_tile_y, rows-2), 0);
	
	// Reuse and retask as appropriate
	TileView* newTileViews[4] = {nil, nil, nil, nil};	// 0=00=UL, 1=01=UR, 2=10=BL, 3=11=BR
	TileView* oldTileViews[4]; [tiles getObjects:oldTileViews range:NSMakeRange(0, 4)];
	
	// Reuse tiles, as possible
	for (NSInteger i = 0; i <= 3; i++)
	{
		CGFloat ox	= (ul_tile_x+(i&1))*w;
		CGFloat oy	= (ul_tile_y+(i>>1))*h;
		CGRect r	= CGRectMake(ox, oy, w, h);
		
		for (NSInteger j = 0; j <= 3; j++)
		{
			if (oldTileViews[j] && CGRectEqualToRect([oldTileViews[j] frame], r))
			{
				newTileViews[i] = oldTileViews[j];
				oldTileViews[j] = nil;
				
				break;
			}
		}
	}
	
	// Retask tiles, as necessary
	for (NSInteger i = 0; i <= 3; i++)
	{
		if (newTileViews[i]) continue;
		for (NSInteger j = 0; j <= 3; j++)
		{
			if (oldTileViews[j])
			{
				CGFloat ox	= (ul_tile_x+(i&1))*w;
				CGFloat oy	= (ul_tile_y+(i>>1))*h;
				
				newTileViews[i] = oldTileViews[j];
				oldTileViews[j] = nil;
				
				newTileViews[i].frame = CGRectMake(ox, oy, w, h);
				[newTileViews[i] setNeedsDisplay];
				
				break;
			}
		}
	}
}

#pragma mark Extension methods

- (void)removeTiles
{
	for (TileView* tv in tiles)
	{
		// Remove tiles from scroll view
		[tv removeFromSuperview];
	}
	[tiles release];
	tiles = nil;
}


- (void)createTiles
{
	if (tiles) [self removeTiles];
	
	TileView* a[4];
	for (NSInteger i = 0; i <= 3; i++)
	{
		a[i] = [[TileView new] autorelease];
		
		a[i].backgroundColor = [UIColor grayColor];
		
		[self.scrollView addSubview:a[i]];
	}
	tiles = [[NSMutableArray alloc] initWithObjects:a count:4];
}

@end
